/*
 * This code is adapted from https://git.karx.xyz/lemonsh/zdiu/src/branch/master/src/discord.rs
 * Licensed under the MIT License
 * */

use multipart::client::lazy::Multipart;
use std::error::Error;
use std::fmt::{Display, Formatter};
use std::io;
use std::io::Read;

#[derive(Debug)]
struct JsonError(String);

impl Display for JsonError {
    fn fmt(&self, f: &mut Formatter<'_>) -> std::fmt::Result {
        write!(f, "Discord AIP sent invalid JSON: {}", self.0)
    }
}

impl JsonError {
    fn new(s: String) -> Self {
        Self(s)
    }
}

impl Error for JsonError {}

struct CountingRead<'a, T, F> {
    inner_stream: &'a mut T,
    current_prog: u64,
    progress_callback: F,
}

impl<'a, T: Read, F: FnMut(u64)> CountingRead<'a, T, F> {
    fn new(inner_stream: &'a mut T, progress_callback: F) -> Self {
        Self {
            inner_stream,
            current_prog: 0,
            progress_callback,
        }
    }
}

impl<T: Read, F: FnMut(u64)> Read for CountingRead<'_, T, F> {
    fn read(&mut self, buf: &mut [u8]) -> io::Result<usize> {
        let res = self.inner_stream.read(buf);
        if let Ok(bytes) = res {
            self.current_prog += bytes as u64;
            (self.progress_callback)(self.current_prog);
        }

        res
    }
}

pub fn upload<F: FnMut(f64)>(
    filename: &str,
    data: &[u8],
    webhook: &str,
    mut callback: F,
) -> anyhow::Result<String> {
    let mut cursor = io::Cursor::new(data);
    let mut mp = Multipart::new();
    let counting_reader = CountingRead::new(&mut cursor, move |bytes| {
        (callback)(bytes as f64 / data.len() as f64);
    });
    mp.add_stream("file", counting_reader, Some(filename.to_string()), None);
    let mpdata = mp.prepare()?;
    let response = ureq::post(webhook)
        .set(
            "Content-Type",
            &format!("multipart/form-data; boundary={}", mpdata.boundary()),
        )
        .send(mpdata)?;
    let response_string = response.into_string()?;
    let mut rjson = json::parse(&response_string)?;
    Ok(rjson["attachments"][0]["url"]
        .take_string()
        .ok_or_else(|| JsonError::new(response_string.clone()))?)
}
